package com.tzld.piaoquan.recommend.server.service.recall.strategy;

import com.tzld.piaoquan.recommend.server.common.RedisKeyConstants;
import com.tzld.piaoquan.recommend.server.model.Video;
import com.tzld.piaoquan.recommend.server.service.filter.FilterParam;
import com.tzld.piaoquan.recommend.server.service.filter.FilterResult;
import com.tzld.piaoquan.recommend.server.service.filter.FilterService;
import com.tzld.piaoquan.recommend.server.service.recall.FilterParamFactory;
import com.tzld.piaoquan.recommend.server.service.recall.RecallParam;
import com.tzld.piaoquan.recommend.server.service.recall.RecallStrategy;
import com.tzld.piaoquan.recommend.server.util.RecallUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ZSetOperations;
import org.springframework.stereotype.Component;

import java.util.*;

@Slf4j
@Component
public class DouHotFlowPoolRecallStrategy implements RecallStrategy {

    @Autowired
    @Qualifier("redisTemplate")
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private FilterService filterService;

    @Value("${dou.hot.recall.video.count:20}")
    private Integer douHotRecallVideoCnt;

    public static final String PUSH_FROM = "recall_strategy_hotspot";

    @Override
    public List<Video> recall(RecallParam param) {
        if (StringUtils.isBlank(param.getProvince())) {
            return Collections.emptyList();
        }

        String redisKey = String.format(RedisKeyConstants.DouHot.ITEM_REDIS_KEY_FORMAT, RecallUtils.douHotProvinceConvert(param.getProvince()), "1");
        log.info("[DouHot recall redisKey] {}", redisKey);
        Set<ZSetOperations.TypedTuple<String>> redisValues = redisTemplate.opsForZSet().rangeWithScores(redisKey, 0, 10000000);


        if (CollectionUtils.isEmpty(redisValues)) {
            log.info("[DouHot recall is empty]");
            return Collections.emptyList();
        }

        Map<Long, String> videoIdAndFlowPoolMap = new HashMap<>(redisValues.size());

        for (ZSetOperations.TypedTuple<String> redisValue : redisValues) {
            String value = redisValue.getValue();
            if (StringUtils.isBlank(value)) {
                continue;
            }
            String[] split = value.split("-");
            if (split.length != 2) {
                continue;
            }
            videoIdAndFlowPoolMap.put(Long.valueOf(split[0]), split[1]);
        }

        FilterParam filterParam = FilterParamFactory.create(param, new ArrayList<>(videoIdAndFlowPoolMap.keySet()));
        FilterResult filterResult = filterService.filter(filterParam);

        // 对视频随机打断
        Collections.shuffle(filterResult.getVideoIds());

        // 截取前20个
        int size = Math.min(filterResult.getVideoIds().size(), douHotRecallVideoCnt);
        List<Long> subList = filterResult.getVideoIds().subList(0, size);
        List<Video> videos = new ArrayList<>(subList.size());
        for (Long videoId : subList) {
            Video video = new Video();
            video.setVideoId(videoId);
            video.setFlowPool(videoIdAndFlowPoolMap.get(videoId));
            video.setPushFrom(pushFrom());
            // 热点宝供给目前只有一层，所以写死
            video.setLevel("1");
            videos.add(video);
        }

        return videos;
    }

    @Override
    public String pushFrom() {
        return PUSH_FROM;
    }
}
